@@ -27,6 +27,8 @@
 #include <linux/interrupt.h>
 #include <linux/io.h>
 #include <linux/irq.h>
+#include <linux/of.h>
+#include <linux/of_gpio.h>
 #include <linux/list.h>
 #include <linux/mfd/core.h>
 #include <linux/mutex.h>
@@ -36,7 +38,6 @@
 #include <linux/spinlock.h>
 #include <linux/workqueue.h>
 
-#include <linux/i2c-tegra.h>
 #include <mach/clk.h>
 #include <mach/iomap.h>
 
@@ -48,7 +49,7 @@
 #define I2C_CNFG_DEBOUNCE_CNT_SHIFT	12
 
 #define I2C_SL_CNFG		0x20
-#define I2C_SL_NEWSL		(1<<2)
+#define I2C_SL_NEWL		(1<<2)
 #define I2C_SL_NACK		(1<<1)
 #define I2C_SL_RESP		(1<<0)
 #define I2C_SL_IRQ		(1<<3)
@@ -78,7 +79,7 @@ static const unsigned char EC_GET_FIRMWARE_VERSION[2]    = "\x07\x15";
 
 /**
  * nvec_power_handle - holds the handle (pointer to a nvec_chip struct) of
- *                     of the ec responsible for powering off
+ *		       of the ec responsible for powering off
  */
 static struct nvec_chip *nvec_power_handle;
 
@@ -272,8 +273,10 @@ struct nvec_msg *nvec_write_sync(struct nvec_chip *nvec,
 
 	nvec->sync_write_pending = (data[1] << 8) + data[0];
 
-	if (nvec_write_async(nvec, data, size) < 0)
+	if (nvec_write_async(nvec, data, size) < 0) {
+		mutex_unlock(&nvec->sync_write_mutex);
 		return NULL;
+	}
 
 	dev_dbg(nvec->dev, "nvec_sync_write: 0x%04x\n",
 					nvec->sync_write_pending);
@@ -656,6 +659,38 @@ static irqreturn_t nvec_interrupt(int irq, void *dev)
 	return IRQ_HANDLED;
 }
 
+static void tegra_init_i2c_slave(struct nvec_chip *nvec)
+{
+	u32 val;
+
+	clk_enable(nvec->i2c_clk);
+
+	tegra_periph_reset_assert(nvec->i2c_clk);
+	udelay(2);
+	tegra_periph_reset_deassert(nvec->i2c_clk);
+
+	val = I2C_CNFG_NEW_MASTER_SFM | I2C_CNFG_PACKET_MODE_EN |
+	    (0x2 << I2C_CNFG_DEBOUNCE_CNT_SHIFT);
+	writel(val, nvec->base + I2C_CNFG);
+
+	clk_set_rate(nvec->i2c_clk, 8 * 80000);
+
+	writel(I2C_SL_NEWL, nvec->base + I2C_SL_CNFG);
+	writel(0x1E, nvec->base + I2C_SL_DELAY_COUNT);
+
+	writel(nvec->i2c_addr>>1, nvec->base + I2C_SL_ADDR1);
+	writel(0, nvec->base + I2C_SL_ADDR2);
+
+	enable_irq(nvec->irq);
+}
+
+static void nvec_disable_i2c_slave(struct nvec_chip *nvec)
+{
+	disable_irq(nvec->irq);
+	writel(I2C_SL_NEWL | I2C_SL_NACK, nvec->base + I2C_SL_CNFG);
+	clk_disable(nvec->i2c_clk);
+}
+
 static void nvec_power_off(void)
 {
 	nvec_write_async(nvec_power_handle, EC_DISABLE_EVENT_REPORTING, 3);
@@ -665,46 +700,83 @@ static void nvec_power_off(void)
 static int __devinit tegra_nvec_probe(struct platform_device *pdev)
 {
 	int err, ret;
+	struct clk *i2c_clk;
 	struct nvec_platform_data *pdata = pdev->dev.platform_data;
 	struct nvec_chip *nvec;
 	struct nvec_msg *msg;
-	struct i2c_adapter *adap;
-	struct tegra_i2c_bus *i2c_bus;
-	struct tegra_i2c_dev *i2c_dev;
+	struct resource *res;
+	struct resource *iomem;
+	void __iomem *base;
 
 	nvec = kzalloc(sizeof(struct nvec_chip), GFP_KERNEL);
 	if (nvec == NULL) {
 		dev_err(&pdev->dev, "failed to reserve memory\n");
 		return -ENOMEM;
 	}
-
 	platform_set_drvdata(pdev, nvec);
 	nvec->dev = &pdev->dev;
-	nvec->gpio = pdata->gpio;
-	nvec->nvec_devices = pdata->nvec_devices;
-	nvec->nr_nvec_devs = pdata->nr_nvec_devs;
 
-	adap = i2c_get_adapter(pdata->adapter);
-	if (adap == NULL) {
-		dev_err(nvec->dev, "failed to get i2c adapter\n");
+	if (pdata) {
+		nvec->gpio = pdata->gpio;
+		nvec->i2c_addr = pdata->i2c_addr;
+		nvec->nvec_devices = pdata->nvec_devices;
+		nvec->nr_nvec_devs = pdata->nr_nvec_devs;
+	} else if (nvec->dev->of_node) {
+		nvec->gpio = of_get_named_gpio(nvec->dev->of_node, "request-gpios", 0);
+		if (nvec->gpio < 0) {
+			dev_err(&pdev->dev, "no gpio specified");
+			goto failed;
+		}
+		if (of_property_read_u32(nvec->dev->of_node, "slave-addr", &nvec->i2c_addr)) {
+			dev_err(&pdev->dev, "no i2c address specified");
+			goto failed;
+		}
+	} else {
+		dev_err(&pdev->dev, "no platform data\n");
+		goto failed;
+	}
+
+	res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
+	if (!res) {
+		dev_err(&pdev->dev, "no mem resource?\n");
 		return -ENODEV;
 	}
-	i2c_bus = i2c_get_adapdata(adap);
-	if (i2c_bus == NULL) {
-		dev_err(nvec->dev, "failed to get i2c bus\n");
-		return -ENODEV;
-	}
-	i2c_dev = i2c_bus->dev;
 
-	nvec->base = i2c_dev->base;
-	nvec->irq = i2c_dev->irq;
-	nvec->clk = i2c_dev->clk;
-	nvec->i2c_addr = i2c_dev->slave_addr;
+	iomem = request_mem_region(res->start, resource_size(res), pdev->name);
+	if (!iomem) {
+		dev_err(&pdev->dev, "I2C region already claimed\n");
+		return -EBUSY;
+	}
+
+	base = ioremap(iomem->start, resource_size(iomem));
+	if (!base) {
+		dev_err(&pdev->dev, "Can't ioremap I2C region\n");
+		return -ENOMEM;
+	}
+
+	res = platform_get_resource(pdev, IORESOURCE_IRQ, 0);
+	if (!res) {
+		dev_err(&pdev->dev, "no irq resource?\n");
+		ret = -ENODEV;
+		goto err_iounmap;
+	}
+
+	i2c_clk = clk_get(&pdev->dev, NULL);
+	if (IS_ERR(i2c_clk)) {
+		dev_err(nvec->dev, "failed to get bus clock\n");
+		ret = PTR_ERR(i2c_clk);
+		goto err_iounmap;
+	}
+
+	nvec->base = base;
+	nvec->irq = res->start;
+	nvec->i2c_clk = i2c_clk;
 	nvec->rx = &nvec->msg_pool[0];
 
-	dev_info(&pdev->dev, "using adapter %s.%d\n", adap->name, i2c_dev->cont_id);
-	dev_info(&pdev->dev, "slave at i2c address 0x%x using irq 0x%x\n",
-		 i2c_dev->slave_addr, nvec->gpio);
+	/* Set the gpio to low when we've got something to say */
+	err = gpio_request(nvec->gpio, "nvec gpio");
+	if (err < 0)
+		dev_err(nvec->dev, "couldn't request gpio\n");
 
 	ATOMIC_INIT_NOTIFIER_HEAD(&nvec->notifier_list);
 
@@ -719,19 +791,17 @@ static int __devinit tegra_nvec_probe(struct platform_device *pdev)
 	INIT_WORK(&nvec->tx_work, nvec_request_master);
 	nvec->wq = alloc_workqueue("nvec", WQ_NON_REENTRANT, 2);
 
-	err = gpio_request_one(nvec->gpio, GPIOF_OUT_INIT_HIGH, "nvec gpio");
-	if (err < 0) {
-		dev_err(nvec->dev, "couldn't request gpio\n");
-		goto failed;
-	}
-
 	err = request_irq(nvec->irq, nvec_interrupt, 0, "nvec", nvec);
 	if (err) {
 		dev_err(nvec->dev, "couldn't request irq\n");
 		goto failed;
 	}
+	disable_irq(nvec->irq);
 
-	clk_enable(nvec->clk);
+	tegra_init_i2c_slave(nvec);
+
+	gpio_direction_output(nvec->gpio, 1);
+	gpio_set_value(nvec->gpio, 1);
 
 	/* enable event reporting */
 	nvec_write_async(nvec, EC_ENABLE_EVENT_REPORTING,
@@ -757,16 +827,19 @@ static int __devinit tegra_nvec_probe(struct platform_device *pdev)
 	}
 
 	if (nvec->nvec_devices) {
-		ret = mfd_add_devices(nvec->dev, -1, nvec->nvec_devices, nvec->nr_nvec_devs,
-				nvec->base, 0);
+		ret = mfd_add_devices(nvec->dev, -1, nvec->nvec_devices,
+				nvec->nr_nvec_devs, nvec->base, 0);
 		if (ret)
 			dev_err(nvec->dev, "error adding subdevices\n");
 		else
-			dev_info(nvec->dev, "added %d NVEC devices\n", nvec->nr_nvec_devs);
+			dev_info(nvec->dev, "added %d NVEC subdevices\n",
+				 nvec->nr_nvec_devs);
 	}
 
 	return 0;
 
+err_iounmap:
+	iounmap(base);
 failed:
 	kfree(nvec);
 	return -ENOMEM;
@@ -793,19 +866,29 @@ static int tegra_nvec_suspend(struct platform_device *pdev, pm_message_t state)
 {
 	struct nvec_chip *nvec = platform_get_drvdata(pdev);
 	struct nvec_msg *msg;
+	int ret = 0;
+
 	dev_dbg(nvec->dev, "suspending\n");
 
 	/* keep these sync or you'll break suspend */
 	msg = nvec_write_sync(nvec, EC_DISABLE_EVENT_REPORTING, 3);
+	if (!msg) {
+		ret = -1;
+		goto fail;
+	}
 	nvec_msg_free(nvec, msg);
 	msg = nvec_write_sync(nvec, "\x04\x02", 2);
+	if (!msg) {
+		ret = -1;
+		goto fail;
+	}
 	nvec_msg_free(nvec, msg);
+fail:
+	if (ret)
+		pr_err("nvec failed in suspend - probalby ec timeout\n");
 
-	disable_irq(nvec->irq);
-	writel(I2C_SL_NEWSL | I2C_SL_NACK, nvec->base + I2C_SL_CNFG);
-	clk_disable(nvec->clk);
-
-	return 0;
+	nvec_disable_i2c_slave(nvec);
+	return ret;
 }
 
 static int tegra_nvec_resume(struct platform_device *pdev)
@@ -813,11 +896,7 @@ static int tegra_nvec_resume(struct platform_device *pdev)
 	struct nvec_chip *nvec = platform_get_drvdata(pdev);
 
 	dev_dbg(nvec->dev, "resuming\n");
-
-	clk_enable(nvec->clk);
-	enable_irq(nvec->irq);
-
-	/* when making the next line nvec_write_sync, resume seems to stop working */
+	tegra_init_i2c_slave(nvec);
 	nvec_write_async(nvec, EC_ENABLE_EVENT_REPORTING, 3);
 
 	return 0;
@@ -828,6 +907,13 @@ static int tegra_nvec_resume(struct platform_device *pdev)
 #define tegra_nvec_resume NULL
 #endif
 
+/* Match table for of_platform binding */
+static const struct of_device_id nvidia_nvec_of_match[] __devinitconst = {
+	{ .compatible = "nvidia,nvec", },
+	{},
+};
+MODULE_DEVICE_TABLE(of, nvidia_nvec_of_match);
+
 static struct platform_driver nvec_device_driver = {
 	.probe   = tegra_nvec_probe,
 	.remove  = __devexit_p(tegra_nvec_remove),
@@ -836,6 +922,7 @@ static struct platform_driver nvec_device_driver = {
 	.driver  = {
 		.name = "nvec",
 		.owner = THIS_MODULE,
+		.of_match_table = nvidia_nvec_of_match,
 	}
 };
 
