@@ -18,6 +18,7 @@
 #include <linux/serio.h>
 #include <linux/delay.h>
 #include <linux/platform_device.h>
+#include <linux/earlysuspend.h>
 
 #include "nvec.h"
 
@@ -27,7 +28,8 @@
 
 #ifdef NVEC_PS2_DEBUG
 #define NVEC_PHD(str, buf, len) \
-	print_hex_dump(KERN_DEBUG, str, DUMP_PREFIX_NONE, 16, 1, buf, len, false)
+	print_hex_dump(KERN_DEBUG, str, DUMP_PREFIX_NONE, \
+			16, 1, buf, len, false)
 #else
 #define NVEC_PHD(str, buf, len)
 #endif
@@ -38,6 +40,9 @@ struct nvec_ps2 {
 	struct serio *ser_dev;
 	struct notifier_block notifier;
 	struct nvec_chip *nvec;
+#ifdef CONFIG_HAS_EARLYSUSPEND
+	struct early_suspend early_suspend;
+#endif
 };
 
 static struct nvec_ps2 ps2_dev;
@@ -97,9 +102,9 @@ static int __devinit nvec_mouse_probe(struct platform_device *pdev)
 	struct nvec_chip *nvec = dev_get_drvdata(pdev->dev.parent);
 	struct serio *ser_dev = kzalloc(sizeof(struct serio), GFP_KERNEL);
 
-	ser_dev->id.type = SERIO_8042;
+	ser_dev->id.type = SERIO_PS_PSTHRU;
 	ser_dev->write = ps2_sendcommand;
-	ser_dev->start= ps2_startstreaming;
+	ser_dev->start = ps2_startstreaming;
 	ser_dev->stop = ps2_stopstreaming;
 
 	strlcpy(ser_dev->name, "nvec mouse", sizeof(ser_dev->name));
@@ -118,31 +123,61 @@ static int __devinit nvec_mouse_probe(struct platform_device *pdev)
 	return 0;
 }
 
+#ifdef CONFIG_HAS_EARLYSUSPEND
+static void nvec_mouse_early_suspend(struct early_suspend *h)
+{
+	/* disable mouse */
+	ps2_sendcommand(ps2_dev.ser_dev, 0xf5);
+}
+
+static void nvec_mouse_late_resume(struct early_suspend *h)
+{
+	/* enable mouse */
+	ps2_sendcommand(ps2_dev.ser_dev, 0xf4);
+}
+
 static int nvec_mouse_suspend(struct platform_device *pdev, pm_message_t state)
 {
-	struct nvec_chip *nvec = dev_get_drvdata(pdev->dev.parent);
-
-	/* disable mouse */
-	nvec_write_async(nvec, "\x06\xf4", 2);
-
 	/* send cancel autoreceive */
-	nvec_write_async(nvec, "\x06\x04", 2);
+	ps2_stopstreaming(ps2_dev.ser_dev);
 
 	return 0;
 }
 
 static int nvec_mouse_resume(struct platform_device *pdev)
 {
-	struct nvec_chip *nvec = dev_get_drvdata(pdev->dev.parent);
-
+	/* send start autoreceive */
 	ps2_startstreaming(ps2_dev.ser_dev);
 
-	/* enable mouse */
-	nvec_write_async(nvec, "\x06\xf5", 2);
-
 	return 0;
 }
 
+#else
+
+static int nvec_mouse_suspend(struct platform_device *pdev, pm_message_t state)
+{
+	/* disable mouse */
+	ps2_sendcommand(ps2_dev.ser_dev, 0xf5);
+
+	/* send cancel autoreceive */
+	ps2_stopstreaming(ps2_dev.ser_dev);
+
+	return 0;
+}
+
+static int nvec_mouse_resume(struct platform_device *pdev)
+{
+	/* send cancel autoreceive */
+	ps2_startstreaming(ps2_dev.ser_dev);
+
+	/* enable mouse */
+	ps2_sendcommand(ps2_dev.ser_dev, 0xf4);
+
+	return 0;
+}
+
+#endif /* HAS_EARLY_SUSPEND */
+
 static struct platform_driver nvec_mouse_driver = {
 	.probe  = nvec_mouse_probe,
 	.suspend = nvec_mouse_suspend,
@@ -155,6 +190,13 @@ static struct platform_driver nvec_mouse_driver = {
 
 static int __init nvec_mouse_init(void)
 {
+#ifdef CONFIG_HAS_EARLYSUSPEND
+	ps2_dev.early_suspend.level = EARLY_SUSPEND_LEVEL_BLANK_SCREEN;
+	ps2_dev.early_suspend.suspend = nvec_mouse_early_suspend;
+	ps2_dev.early_suspend.resume = nvec_mouse_late_resume;
+	register_early_suspend(&ps2_dev.early_suspend);
+#endif
+
 	return platform_driver_register(&nvec_mouse_driver);
 }
 
