@@ -22,6 +22,7 @@
 #include <linux/pm.h>
 #include <linux/i2c.h>
 #include <linux/slab.h>
+#include <linux/regmap.h>
 #include <sound/core.h>
 #include <sound/pcm.h>
 #include <sound/pcm_params.h>
@@ -34,45 +35,59 @@
 /*
  * ALC5632 register cache
  */
-static const u16 alc5632_reg_defaults[] = {
-	0x59B4, 0x0000, 0x8080, 0x0000, /* 0 */
-	0x8080, 0x0000, 0x8080, 0x0000, /* 4 */
-	0xC800, 0x0000, 0xE808, 0x0000, /* 8 */
-	0x1010, 0x0000, 0x0808, 0x0000, /* 12 */
-	0xEE0F, 0x0000, 0xCBCB, 0x0000, /* 16 */
-	0x7F7F, 0x0000, 0x0000, 0x0000, /* 20 */
-	0xE010, 0x0000, 0x0000, 0x0000, /* 24 */
-	0x8008, 0x0000, 0x0000, 0x0000, /* 28 */
-	0x0000, 0x0000, 0x0000, 0x0000, /* 32 */
-	0x00C0, 0x0000, 0xEF00, 0x0000, /* 36 */
-	0x0000, 0x0000, 0x0000, 0x0000, /* 40 */
-	0x0000, 0x0000, 0x0000, 0x0000, /* 44 */
-	0x0000, 0x0000, 0x0000, 0x0000, /* 48 */
-	0x8000, 0x0000, 0x0000, 0x0000, /* 52 */
-	0x0000, 0x0000, 0x0000, 0x0000, /* 56 */
-	0x0000, 0x0000, 0x8000, 0x0000, /* 60 */
-	0x0C0A, 0x0000, 0x0000, 0x0000, /* 64 */
-	0x0000, 0x0000, 0x0000, 0x0000, /* 68 */
-	0x0000, 0x0000, 0x0000, 0x0000, /* 72 */
-	0xBE3E, 0x0000, 0xBE3E, 0x0000, /* 76 */
-	0x0000, 0x0000, 0x0000, 0x0000, /* 80 */
-	0x803A, 0x0000, 0x0000, 0x0000, /* 84 */
-	0x0000, 0x0000, 0x0009, 0x0000, /* 88 */
-	0x0000, 0x0000, 0x3000, 0x0000, /* 92 */
-	0x3075, 0x0000, 0x1010, 0x0000, /* 96 */
-	0x3110, 0x0000, 0x0000, 0x0000, /* 100 */
-	0x0553, 0x0000, 0x0000, 0x0000, /* 104 */
-	0x0000, 0x0000, 0x0000, 0x0000, /* 108 */
+static struct reg_default  alc5632_reg_defaults[] = {
+	{   2, 0x8080 },	/* R2   - Speaker Output Volume */
+	{   4, 0x8080 },	/* R4   - Headphone Output Volume */
+	{   6, 0x8080 },	/* R6   - AUXOUT Volume */
+	{   8, 0xC800 },	/* R8   - Phone Input */
+	{  10, 0xE808 },	/* R10  - LINE_IN Volume */
+	{  12, 0x1010 },	/* R12  - STEREO DAC Input Volume */
+	{  14, 0x0808 },	/* R14  - MIC Input Volume */
+	{  16, 0xEE0F },	/* R16  - Stereo DAC and MIC Routing Control */
+	{  18, 0xCBCB },	/* R18  - ADC Record Gain */
+	{  20, 0x7F7F },	/* R20  - ADC Record Mixer Control */
+	{  24, 0xE010 },	/* R24  - Voice DAC Volume */
+	{  28, 0x8008 },	/* R28  - Output Mixer Control */
+	{  34, 0x0000 },	/* R34  - Microphone Control */
+	{  36, 0x00C0 },    /* R36  - Codec Digital MIC/Digital Boost
+						   Control */
+	{  46, 0x0000 },	/* R46  - Stereo DAC/Voice DAC/Stereo ADC
+						   Function Select */
+	{  52, 0x8000 },	/* R52  - Main Serial Data Port Control
+						   (Stereo I2S) */
+	{  54, 0x0000 },	/* R54  - Extend Serial Data Port Control
+						   (VoDAC_I2S/PCM) */
+	{  58, 0x0000 },	/* R58  - Power Management Addition 1 */
+	{  60, 0x0000 },	/* R60  - Power Management Addition 2 */
+	{  62, 0x8000 },	/* R62  - Power Management Addition 3 */
+	{  64, 0x0C0A },	/* R64  - General Purpose Control Register 1 */
+	{  66, 0x0000 },	/* R66  - General Purpose Control Register 2 */
+	{  68, 0x0000 },	/* R68  - PLL1 Control */
+	{  70, 0x0000 },	/* R70  - PLL2 Control */
+	{  76, 0xBE3E },	/* R76  - GPIO Pin Configuration */
+	{  78, 0xBE3E },	/* R78  - GPIO Pin Polarity */
+	{  80, 0x0000 },	/* R80  - GPIO Pin Sticky */
+	{  82, 0x0000 },	/* R82  - GPIO Pin Wake Up */
+	{  86, 0x0000 },	/* R86  - Pin Sharing */
+	{  90, 0x0009 },	/* R90  - Soft Volume Control Setting */
+	{  92, 0x0000 },	/* R92  - GPIO_Output Pin Control */
+	{  94, 0x3000 },	/* R94  - MISC Control */
+	{  96, 0x3075 },	/* R96  - Stereo DAC Clock Control_1 */
+	{  98, 0x1010 },	/* R98  - Stereo DAC Clock Control_2 */
+	{ 100, 0x3110 },	/* R100 - VoDAC_PCM Clock Control_1 */
+	{ 104, 0x0553 },	/* R104 - Pseudo Stereo and Spatial Effect
+						   Block Control */
+	{ 106, 0x0000 },	/* R106 - Private Register Address */
 };
 
 /* codec private data */
 struct alc5632_priv {
-	enum snd_soc_control_type control_type;
+	struct regmap *regmap;
 	u8 id;
 	unsigned int sysclk;
 };
 
-static int alc5632_volatile_register(struct snd_soc_codec *codec,
+static bool alc5632_volatile_register(struct device *dev,
 							unsigned int reg)
 {
 	switch (reg) {
@@ -84,18 +99,18 @@ static int alc5632_volatile_register(struct snd_soc_codec *codec,
 	case ALC5632_EQ_CTRL:
 	case ALC5632_VENDOR_ID1:
 	case ALC5632_VENDOR_ID2:
-		return 1;
+		return true;
 
 	default:
 		break;
 	}
 
-	return 0;
+	return false;
 }
 
-static inline int alc5632_reset(struct snd_soc_codec *codec)
+static inline int alc5632_reset(struct regmap *map)
 {
-	return snd_soc_write(codec, ALC5632_RESET, 0);
+	return regmap_write(map, ALC5632_RESET, 0x59B4);
 }
 
 static int amp_mixer_event(struct snd_soc_dapm_widget *w,
@@ -262,19 +277,19 @@ static const struct snd_kcontrol_new alc5632_dmicr_mixer_controls[] = {
 SOC_DAPM_SINGLE("DMICR2ADC Capture Switch", ALC5632_DIGI_BOOST_CTRL, 6, 1, 1),
 };
 
-static const char *alc5632_spk_n_sour_sel[] = {
+static const char * const alc5632_spk_n_sour_sel[] = {
 		"RN/-R", "RP/+R", "LN/-R", "Mute"};
-static const char *alc5632_hpl_out_input_sel[] = {
+static const char * const alc5632_hpl_out_input_sel[] = {
 		"Vmid", "HP Left Mix"};
-static const char *alc5632_hpr_out_input_sel[] = {
+static const char * const alc5632_hpr_out_input_sel[] = {
 		"Vmid", "HP Right Mix"};
-static const char *alc5632_spkout_input_sel[] = {
+static const char * const alc5632_spkout_input_sel[] = {
 		"Vmid", "HPOut Mix", "Speaker Mix", "Mono Mix"};
-static const char *alc5632_aux_out_input_sel[] = {
+static const char * const alc5632_aux_out_input_sel[] = {
 		"Vmid", "HPOut Mix", "Speaker Mix", "Mono Mix"};
-static const char *alc5632_adcr_func_sel[] = {
+static const char * const alc5632_adcr_func_sel[] = {
 		"Stereo ADC", "Voice ADC"};
-static const char *alc5632_i2s_out_sel[] = {
+static const char * const alc5632_i2s_out_sel[] = {
 		"ADC LR", "Voice Stereo Digital"};
 
 /* auxout output mux */
@@ -340,7 +355,7 @@ SND_SOC_DAPM_MUX("SpeakerOut N Mux", SND_SOC_NOPM, 0, 0,
 	&alc5632_spkoutn_mux_controls),
 SND_SOC_DAPM_MUX("ADCR Mux", SND_SOC_NOPM, 0, 0,
 	&alc5632_adcr_func_controls),
-SND_SOC_DAPM_MUX("I2SOut Mux", SND_SOC_NOPM, 0, 0,
+SND_SOC_DAPM_MUX("I2SOut Mux", ALC5632_PWR_MANAG_ADD1, 11, 0,
 	&alc5632_i2s_out_controls),
 
 /* output mixers */
@@ -391,7 +406,7 @@ SND_SOC_DAPM_ADC("Right ADC", NULL, ALC5632_PWR_MANAG_ADD2, 6, 0),
 SND_SOC_DAPM_MIXER("DAC Left Channel", ALC5632_PWR_MANAG_ADD1, 15, 0, NULL, 0),
 SND_SOC_DAPM_MIXER("DAC Right Channel",
 	ALC5632_PWR_MANAG_ADD1, 14, 0, NULL, 0),
-SND_SOC_DAPM_MIXER("I2S Mix", SND_SOC_NOPM, 0, 0, NULL, 0),
+SND_SOC_DAPM_MIXER("I2S Mix", ALC5632_PWR_MANAG_ADD1, 11, 0, NULL, 0),
 SND_SOC_DAPM_MIXER("Phone Mix", SND_SOC_NOPM, 0, 0, NULL, 0),
 SND_SOC_DAPM_MIXER("Line Mix", SND_SOC_NOPM, 0, 0, NULL, 0),
 SND_SOC_DAPM_MIXER("Voice Mix", SND_SOC_NOPM, 0, 0, NULL, 0),
@@ -412,7 +427,6 @@ SND_SOC_DAPM_PGA("MIC1 Pre Amp", ALC5632_PWR_MANAG_ADD3, 1, 0, NULL, 0),
 SND_SOC_DAPM_PGA("MIC2 Pre Amp", ALC5632_PWR_MANAG_ADD3, 0, 0, NULL, 0),
 SND_SOC_DAPM_SUPPLY("MICBIAS1", ALC5632_PWR_MANAG_ADD1, 3, 0, NULL, 0),
 SND_SOC_DAPM_SUPPLY("MICBIAS2", ALC5632_PWR_MANAG_ADD1, 2, 0, NULL, 0),
-SND_SOC_DAPM_SUPPLY("I2S Pwr", ALC5632_PWR_MANAG_ADD1, 11, 0, NULL, 0),
 
 SND_SOC_DAPM_PGA_E("D Amp", ALC5632_PWR_MANAG_ADD2, 14, 0, NULL, 0,
 	amp_mixer_event, SND_SOC_DAPM_PRE_PMU | SND_SOC_DAPM_POST_PMD),
@@ -443,7 +457,6 @@ static const struct snd_soc_dapm_route alc5632_dapm_routes[] = {
 	{"Right DAC", NULL, "AIFRXR"},
 
 	/* virtual mixer - mixes left & right channels */
-	{"I2S Mix",	NULL,	"I2S Pwr"},
 	{"I2S Mix",	NULL,	"Left DAC"},
 	{"I2S Mix",	NULL,	"Right DAC"},
 	{"Line Mix",	NULL,	"Right LineIn"},
@@ -537,29 +550,30 @@ static const struct snd_soc_dapm_route alc5632_dapm_routes[] = {
 	/* input pga */
 	{"Left LineIn", NULL,				"LINEINL"},
 	{"Right LineIn", NULL,				"LINEINR"},
-	{"Phone", NULL,				"PHONEP"},
-	{"MIC1 Pre Amp", NULL,				"MIC1"},
-	{"MIC2 Pre Amp", NULL,				"MIC2"},
+	{"Phone", NULL,					"PHONEP"},
+	{"MICBIAS1", NULL,				"MIC1"},
+	{"MIC1 Pre Amp", NULL,				"MICBIAS1"},
+	{"MICBIAS2", NULL,				"MIC2"},
+	{"MIC2 Pre Amp", NULL,				"MICBIAS2"},
 	{"MIC1 PGA", NULL,				"MIC1 Pre Amp"},
 	{"MIC2 PGA", NULL,				"MIC2 Pre Amp"},
 
 	/* left ADC */
 	{"Left ADC", NULL,				"Left Capture Mix"},
-	{"DMICL Mix", "DMICL2ADC Capture Switch", "DMICDAT"},
+	{"DMICL Mix", "DMICL2ADC Capture Switch",	"DMICDAT"},
 	{"Left ADC", NULL,				"DMICL Mix"},
 	{"ADCLR", NULL,					"Left ADC"},
 
 	/* right ADC */
-	{"Right ADC", NULL, "Right Capture Mix"},
-	{"DMICR Mix", "DMICR2ADC Capture Switch", "DMICDAT"},
-	{"Right ADC", NULL, "DMICR Mix"},
-	{"ADCR Mux", "Stereo ADC", "Right ADC"},
-	{"ADCR Mux", "Voice ADC", "Right ADC"},
-	{"ADCLR", NULL, "ADCR Mux"},
-	{"VAIFTX", NULL, "ADCR Mux"},
+	{"Right ADC", NULL,				"Right Capture Mix"},
+	{"DMICR Mix", "DMICR2ADC Capture Switch",	"DMICDAT"},
+	{"Right ADC", NULL,				"DMICR Mix"},
+	{"ADCR Mux", "Stereo ADC",			"Right ADC"},
+	{"ADCR Mux", "Voice ADC",			"Right ADC"},
+	{"ADCLR", NULL,					"ADCR Mux"},
+	{"VAIFTX", NULL,				"ADCR Mux"},
 
 	/* Digital I2S out */
-	{"I2SOut Mux", NULL, "I2S Pwr"},
 	{"I2SOut Mux", "ADC LR", "ADCLR"},
 	{"I2SOut Mux", "Voice Stereo Digital", "VAIFRX"},
 	{"AIFTXL", NULL, "I2SOut Mux"},
@@ -987,7 +1001,7 @@ static int alc5632_set_bias_level(struct snd_soc_codec *codec,
 			| SNDRV_PCM_FMTBIT_S24_LE \
 			| SNDRV_PCM_FMTBIT_S32_LE)
 
-static struct snd_soc_dai_ops alc5632_dai_ops = {
+static const struct snd_soc_dai_ops alc5632_dai_ops = {
 		.hw_params = alc5632_pcm_hw_params,
 		.digital_mute = alc5632_mute,
 		.set_fmt = alc5632_set_dai_fmt,
@@ -1027,16 +1041,9 @@ static int alc5632_suspend(struct snd_soc_codec *codec, pm_message_t mesg)
 
 static int alc5632_resume(struct snd_soc_codec *codec)
 {
-	int ret;
+	struct alc5632_priv *alc5632 = snd_soc_codec_get_drvdata(codec);
 
-	/* mark cache as needed to sync */
-	codec->cache_sync = 1;
-
-	ret = snd_soc_cache_sync(codec);
-	if (ret != 0) {
-		dev_err(codec->dev, "Failed to sync cache: %d\n", ret);
-		return ret;
-	}
+	regcache_sync(alc5632->regmap);
 
 	alc5632_set_bias_level(codec, SND_SOC_BIAS_STANDBY);
 	return 0;
@@ -1051,15 +1058,11 @@ static int alc5632_probe(struct snd_soc_codec *codec)
 	struct alc5632_priv *alc5632 = snd_soc_codec_get_drvdata(codec);
 	int ret;
 
-	ret = snd_soc_codec_set_cache_io(codec, 8, 16, alc5632->control_type);
-	if (ret < 0) {
-		dev_err(codec->dev, "Failed to set cache I/O: %d\n", ret);
-		return ret;
-	}
+	codec->control_data = alc5632->regmap;
 
-	ret = alc5632_reset(codec);
-	if (ret < 0) {
-		dev_err(codec->dev, "Failed to issue reset\n");
+	ret = snd_soc_codec_set_cache_io(codec, 8, 16, SND_SOC_I2C);
+	if (ret != 0) {
+		dev_err(codec->dev, "Failed to set cache I/O: %d\n", ret);
 		return ret;
 	}
 
@@ -1091,11 +1094,6 @@ static struct snd_soc_codec_driver soc_codec_device_alc5632 = {
 	.suspend = alc5632_suspend,
 	.resume = alc5632_resume,
 	.set_bias_level = alc5632_set_bias_level,
-	.reg_word_size = sizeof(u16),
-	.reg_cache_step = 2,
-	.reg_cache_default = alc5632_reg_defaults,
-	.reg_cache_size = ARRAY_SIZE(alc5632_reg_defaults),
-	.volatile_register = alc5632_volatile_register,
 	.controls = alc5632_snd_controls,
 	.num_controls = ARRAY_SIZE(alc5632_snd_controls),
 	.dapm_widgets = alc5632_dapm_widgets,
@@ -1104,33 +1102,53 @@ static struct snd_soc_codec_driver soc_codec_device_alc5632 = {
 	.num_dapm_routes = ARRAY_SIZE(alc5632_dapm_routes),
 };
 
+static struct regmap_config alc5632_regmap = {
+	.reg_bits = 8,
+	.val_bits = 16,
+
+	.max_register = ALC5632_MAX_REGISTER,
+	.reg_defaults = alc5632_reg_defaults,
+	.num_reg_defaults = ARRAY_SIZE(alc5632_reg_defaults),
+	.volatile_reg = alc5632_volatile_register,
+	.cache_type = REGCACHE_RBTREE,
+};
+
 /*
  * alc5632 2 wire address is determined by A1 pin
  * state during powerup.
  *    low  = 0x1a
  *    high = 0x1b
  */
-static int alc5632_i2c_probe(struct i2c_client *client,
+static __devinit int alc5632_i2c_probe(struct i2c_client *client,
 				const struct i2c_device_id *id)
 {
 	struct alc5632_priv *alc5632;
-	int ret, vid1, vid2;
+	int ret, ret1, ret2;
+	unsigned int vid1, vid2;
 
 	alc5632 = devm_kzalloc(&client->dev,
 			 sizeof(struct alc5632_priv), GFP_KERNEL);
 	if (alc5632 == NULL)
 		return -ENOMEM;
 
-	vid1 = i2c_smbus_read_word_data(client, ALC5632_VENDOR_ID1);
-	vid2 = i2c_smbus_read_word_data(client, ALC5632_VENDOR_ID2);
-	if (vid1 < 0 || vid2 < 0) {
+	i2c_set_clientdata(client, alc5632);
+
+	alc5632->regmap = devm_regmap_init_i2c(client, &alc5632_regmap);
+	if (IS_ERR(alc5632->regmap)) {
+		ret = PTR_ERR(alc5632->regmap);
+		dev_err(&client->dev, "regmap_init() failed: %d\n", ret);
+		return ret;
+	}
+
+	ret1 = regmap_read(alc5632->regmap, ALC5632_VENDOR_ID1, &vid1);
+	ret2 = regmap_read(alc5632->regmap, ALC5632_VENDOR_ID2, &vid2);
+	if (ret1 != 0 || ret2 != 0) {
 		dev_err(&client->dev,
-		"Failed to read chip ID: vid1=%d, vid2=%d\n", vid1, vid2);
+		"Failed to read chip ID: ret1=%d, ret2=%d\n", ret1, ret2);
 		return -EIO;
 	}
 
-	vid1 = ((vid1 & 0xff) << 8) | (vid1 >> 8);
-	vid2 = (vid2 & 0xff);
+	vid2 >>= 8;
 
 	if ((vid1 != 0x10EC) || (vid2 != id->driver_data)) {
 		dev_err(&client->dev,
@@ -1138,6 +1156,12 @@ static int alc5632_i2c_probe(struct i2c_client *client,
 		return -EINVAL;
 	}
 
+	ret = alc5632_reset(alc5632->regmap);
+	if (ret < 0) {
+		dev_err(&client->dev, "Failed to issue reset\n");
+		return ret;
+	}
+
 	alc5632->id = vid2;
 	switch (alc5632->id) {
 	case 0x5c:
@@ -1147,22 +1171,20 @@ static int alc5632_i2c_probe(struct i2c_client *client,
 		return -EINVAL;
 	}
 
-	i2c_set_clientdata(client, alc5632);
-	alc5632->control_type = SND_SOC_I2C;
-
-	ret =  snd_soc_register_codec(&client->dev,
+	ret = snd_soc_register_codec(&client->dev,
 		&soc_codec_device_alc5632, &alc5632_dai, 1);
 
-	if (ret < 0)
+	if (ret < 0) {
 		dev_err(&client->dev, "Failed to register codec: %d\n", ret);
+		return ret;
+	}
 
 	return ret;
 }
 
-static int alc5632_i2c_remove(struct i2c_client *client)
+static __devexit int alc5632_i2c_remove(struct i2c_client *client)
 {
 	snd_soc_unregister_codec(&client->dev);
-
 	return 0;
 }
 
@@ -1183,25 +1205,7 @@ static struct i2c_driver alc5632_i2c_driver = {
 	.id_table = alc5632_i2c_table,
 };
 
-static int __init alc5632_modinit(void)
-{
-	int ret;
-
-	ret = i2c_add_driver(&alc5632_i2c_driver);
-	if (ret != 0) {
-		printk(KERN_ERR "%s: can't add i2c driver", __func__);
-		return ret;
-	}
-
-	return ret;
-}
-module_init(alc5632_modinit);
-
-static void __exit alc5632_modexit(void)
-{
-	i2c_del_driver(&alc5632_i2c_driver);
-}
-module_exit(alc5632_modexit);
+module_i2c_driver(alc5632_i2c_driver);
 
 MODULE_DESCRIPTION("ASoC ALC5632 driver");
 MODULE_AUTHOR("Leon Romanovsky <leon@leon.nu>");
