@@ -29,8 +29,8 @@
 
 #include "../codecs/alc5632.h"
 
-#include "tegra_das.h"
-#include "tegra_i2s.h"
+#include "tegra20_das.h"
+#include "tegra20_i2s.h"
 #include "tegra_pcm.h"
 #include "tegra_asoc_utils.h"
 
@@ -61,10 +61,16 @@ static int tegra_alc5632_asoc_hw_params(struct snd_pcm_substream *substream,
 
 	err = tegra_asoc_utils_set_rate(&alc5632->util_data, srate, mclk);
 	if (err < 0) {
-		dev_err(card->dev, "Can't configure clocks\n");
-		return err;
+		if (!(alc5632->util_data.set_mclk % mclk))
+			mclk = alc5632->util_data.set_mclk;
+		else {
+			dev_err(card->dev, "Can't configure clocks\n");
+			return err;
+		}
 	}
 
+	tegra_asoc_utils_lock_clk_rate(&alc5632->util_data, 1);
+
 	err = snd_soc_dai_set_fmt(codec_dai,
 					SND_SOC_DAIFMT_I2S |
 					SND_SOC_DAIFMT_NB_NF |
@@ -93,8 +99,19 @@ static int tegra_alc5632_asoc_hw_params(struct snd_pcm_substream *substream,
 	return 0;
 }
 
+static int tegra_alc5632_hw_free(struct snd_pcm_substream *substream)
+{
+	struct snd_soc_pcm_runtime *rtd = substream->private_data;
+	struct tegra_alc5632 *alc5632 = snd_soc_card_get_drvdata(rtd->card);
+
+	tegra_asoc_utils_lock_clk_rate(&alc5632->util_data, 0);
+
+	return 0;
+}
+
 static struct snd_soc_ops tegra_alc5632_asoc_ops = {
 	.hw_params = tegra_alc5632_asoc_hw_params,
+	.hw_free = tegra_alc5632_hw_free,
 };
 
 static struct snd_soc_jack tegra_alc5632_hs_jack;
@@ -175,7 +192,7 @@ static struct snd_soc_dai_link tegra_alc5632_dai = {
 	.stream_name = "ALC5632 PCM",
 	.codec_name = "alc5632.0-001e",
 	.platform_name = "tegra-pcm-audio",
-	.cpu_dai_name = "tegra-i2s.0",
+	.cpu_dai_name = "tegra20-i2s.0",
 	.codec_dai_name = "alc5632-hifi",
 	.init = tegra_alc5632_asoc_init,
 	.ops = &tegra_alc5632_asoc_ops,
@@ -220,7 +237,7 @@ static __devinit int tegra_alc5632_probe(struct platform_device *pdev)
 	platform_set_drvdata(pdev, card);
 	snd_soc_card_set_drvdata(card, alc5632);
 
-	ret = tegra_asoc_utils_init(&alc5632->util_data, &pdev->dev);
+	ret = tegra_asoc_utils_init(&alc5632->util_data, &pdev->dev, card);
 	if (ret)
 		goto err;
 
@@ -274,7 +291,18 @@ static struct platform_driver tegra_alc5632_driver = {
 	.probe = tegra_alc5632_probe,
 	.remove = __devexit_p(tegra_alc5632_remove),
 };
-module_platform_driver(tegra_alc5632_driver);
+
+static int __init tegra_alc5632_modinit(void)
+{
+	return platform_driver_register(&tegra_alc5632_driver);
+}
+module_init(tegra_alc5632_modinit);
+
+static void __exit tegra_alc5632_modexit(void)
+{
+	platform_driver_unregister(&tegra_alc5632_driver);
+}
+module_exit(tegra_alc5632_modexit);
 
 MODULE_AUTHOR("Leon Romanovsky <leon@leon.nu>");
 MODULE_DESCRIPTION("Tegra+ALC5632 machine ASoC driver");
